@@ -5,7 +5,13 @@ pbmc_small <- suppressWarnings(UpdateSeuratObject(pbmc_small))
 # Setup test objects
 ref <- pbmc_small
 query <- CreateSeuratObject(
-  counts = GetAssayData(object = pbmc_small[['RNA']], slot = "counts") + rpois(n = ncol(pbmc_small), lambda = 1)
+  counts = as.sparse(
+    GetAssayData(
+      object = pbmc_small[['RNA']],
+      layer = "counts") + rpois(n = ncol(pbmc_small),
+      lambda = 1
+    )
+  )
 )
 query <- NormalizeData(object = query, verbose = FALSE)
 query <- FindVariableFeatures(object = query, verbose = FALSE, nfeatures = 100)
@@ -20,8 +26,8 @@ test_that("FindTransferAnchors defaults work", {
   co <- anchors@object.list[[1]]
   expect_equal(dim(co), c(100, 160))
   expect_equal(Reductions(co), c("pcaproject", "pcaproject.l2"))
-  expect_equal(GetAssayData(co[["RNA"]])[1, 3], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")[1, 3], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer ="data")[1, 3], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")[1, 3], 0)
   expect_equal(dim(co[['pcaproject']]), c(160, 30))
   expect_equal(Embeddings(co[['pcaproject']])[1, 1], 0.4840944592, tolerance = 1e-7)
   expect_equal(Loadings(co[['pcaproject']], projected = T)[1, 1], 0.2103563963, tolerance = 1e-7)
@@ -73,7 +79,7 @@ test_that("FindTransferAnchors allows reference.reduction to be precomputed", {
   expect_equal(dim(co), c(100, 160))
   expect_equal(Reductions(co), c("pcaproject", "pcaproject.l2"))
   expect_equal(GetAssayData(co[["RNA"]])[1, 3], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")[1, 3], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")[1, 3], 0)
   expect_equal(dim(co[['pcaproject']]), c(160, 30))
   expect_equal(Embeddings(co[['pcaproject']])[1, 1], 0.4840944592, tolerance = 1e-7)
   expect_equal(Loadings(co[['pcaproject']], projected = T)[1, 1], 0.2103563963, tolerance = 1e-7)
@@ -103,8 +109,8 @@ test_that("FindTransferAnchors with cca defaults work", {
   expect_equal(Reductions(co), c("cca", "cca.l2"))
   expect_equal(GetAssayData(co[["RNA"]])["PPBP", 3], 0)
   expect_equal(GetAssayData(co[["RNA"]])["PPBP", 1], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")["PPBP", 3], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")["PPBP", 1], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")["PPBP", 3], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")["PPBP", 1], 0)
   expect_equal(dim(co[['cca']]), c(160, 30))
   expect_equal(Embeddings(co[['cca']])[1, 1], 0.04611130861, tolerance = 1e-7)
   expect_equal(Loadings(co[['cca']], projected = T)["PPBP", 1], 12.32379661, tolerance = 1e-7)
@@ -132,10 +138,10 @@ test_that("FindTransferAnchors with project.query defaults work", {
   co <- anchors@object.list[[1]]
   expect_equal(dim(co), c(100, 160))
   expect_equal(Reductions(co), c("pcaproject", "pcaproject.l2"))
-  expect_equal(GetAssayData(co[["RNA"]])["PPBP", 3], 0)
-  expect_equal(GetAssayData(co[["RNA"]])["PPBP", 1], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")["PPBP", 3], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")["PPBP", 1], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "data")["PPBP", 3], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "data")["PPBP", 1], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")["PPBP", 3], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")["PPBP", 1], 0)
   expect_equal(dim(co[['pcaproject']]), c(160, 30))
   expect_equal(Embeddings(co[['pcaproject']])[1, 1], 1.577959404, tolerance = 1e-7)
   expect_equal(Loadings(co[['pcaproject']], projected = T)["PPBP", 1], 0.1145472305, tolerance = 1e-7)
@@ -168,8 +174,8 @@ test_that("FindTransferAnchors with project.query and reference.reduction works"
   expect_equal(Reductions(co), c("pcaproject", "pcaproject.l2"))
   expect_equal(GetAssayData(co[["RNA"]])["PPBP", 3], 0)
   expect_equal(GetAssayData(co[["RNA"]])["PPBP", 1], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")["PPBP", 3], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")["PPBP", 1], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")["PPBP", 3], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")["PPBP", 1], 0)
   expect_equal(dim(co[['pcaproject']]), c(160, 30))
   expect_equal(Embeddings(co[['pcaproject']])[1, 1], 1.577959404, tolerance = 1e-7)
   expect_equal(Loadings(co[['pcaproject']], projected = T)["PPBP", 1], 0.1145472305, tolerance = 1e-7)
@@ -202,7 +208,7 @@ test_that("FindTransferAnchors with reference.neighbors precomputed works", {
   expect_equal(dim(co), c(100, 160))
   expect_equal(Reductions(co), c("pcaproject", "pcaproject.l2"))
   expect_equal(GetAssayData(co[["RNA"]])[1, 3], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")[1, 3], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")[1, 3], 0)
   expect_equal(dim(co[['pcaproject']]), c(160, 30))
   expect_equal(Embeddings(co[['pcaproject']])[1, 1], 0.4840944592, tolerance = 1e-7)
   expect_equal(Loadings(co[['pcaproject']], projected = T)[1, 1], 0.2103563963, tolerance = 1e-7)
@@ -231,7 +237,7 @@ test_that("FindTransferAnchors with no l2 works", {
   expect_equal(dim(co), c(100, 160))
   expect_equal(Reductions(co), c("pcaproject"))
   expect_equal(GetAssayData(co[["RNA"]])[1, 3], 0)
-  expect_equal(GetAssayData(co[["RNA"]], slot = "counts")[1, 3], 0)
+  expect_equal(GetAssayData(co[["RNA"]], layer = "counts")[1, 3], 0)
   expect_equal(dim(co[['pcaproject']]), c(160, 30))
   expect_equal(Embeddings(co[['pcaproject']])[1, 1], 0.4840944592, tolerance = 1e-7)
   expect_equal(Loadings(co[['pcaproject']], projected = T)[1, 1], 0.2103563963, tolerance = 1e-7)
@@ -250,9 +256,9 @@ test_that("FindTransferAnchors with no l2 works", {
   expect_equal(anchors@neighbors, list())
 })
 
-# SCTransform tests
-query <- suppressWarnings(SCTransform(object = query, verbose = FALSE))
-ref <- suppressWarnings(SCTransform(object = ref, verbose = FALSE))
+# SCTransform tests V1
+query <- suppressWarnings(SCTransform(object = query, verbose = FALSE,vst.flavor = 'v1'))
+ref <- suppressWarnings(SCTransform(object = ref, verbose = FALSE,vst.flavor = 'v1'))
 
 test_that("FindTransferAnchors with default SCT works", {
   skip_on_cran()
@@ -261,7 +267,7 @@ test_that("FindTransferAnchors with default SCT works", {
   expect_equal(dim(co), c(220, 160))
   expect_equal(Reductions(co), c("pcaproject", "pcaproject.l2"))
   expect_equal(DefaultAssay(co), "SCT")
-  expect_equal(GetAssayData(co[["SCT"]], slot = "scale.data"), new(Class = "matrix"))
+  expect_equal(GetAssayData(co[["SCT"]], layer = "scale.data"), new(Class = "matrix"))
   expect_equal(GetAssayData(co[["SCT"]])[1, 1], 0)
   expect_equal(dim(co[['pcaproject']]), c(160, 30))
   expect_equal(Embeddings(co[['pcaproject']])[1, 1], -1.852491719, tolerance = 1e-7)
@@ -306,10 +312,10 @@ test_that("FindTransferAnchors with default SCT works", {
   expect_equal(GetAssayData(co[["SCT"]])[1, 1], 0)
   expect_equal(dim(co[['cca']]), c(160, 30))
   expect_equal(Embeddings(co[['cca']])[1, 1], 0.0459135444, tolerance = 1e-7)
-  expect_equal(Loadings(co[['cca']], projected = T)[1, 1], 8.51477973, tolerance = 1e-7)
+  expect_equal(Loadings(co[['cca']], projected = T)["NKG7", 1], 8.51477973, tolerance = 1e-7)
   expect_equal(dim(co[['cca.l2']]), c(160, 30))
   expect_equal(Embeddings(co[['cca.l2']])[1, 1], 0.0625989664, tolerance = 1e-7)
-  expect_equal(Loadings(co[['cca.l2']], projected = T)[1, 1], 8.51477973, tolerance = 1e-7)
+  expect_equal(Loadings(co[['cca.l2']], projected = T)["NKG7", 1], 8.51477973, tolerance = 1e-7)
   ref.cells <- paste0(Cells(ref), "_reference")
   query.cells <- paste0(Cells(query), "_query")
   expect_equal(anchors@reference.cells, ref.cells)
@@ -335,10 +341,10 @@ test_that("FindTransferAnchors with SCT and project.query work", {
   expect_equal(GetAssayData(co[["SCT"]])[1, 1], 0)
   expect_equal(GetAssayData(co[["SCT"]], slot = "scale.data"), new("matrix"))
   expect_equal(dim(co[['pcaproject']]), c(160, 30))
-  expect_equal(Embeddings(co[['pcaproject']])[1, 1], 0.3308694488, tolerance = 1e-7)
+  expect_equal(Embeddings(co[['pcaproject']])[1, 1], 0.3049308, tolerance = 1e-7)
   expect_equal(Loadings(co[['pcaproject']], projected = T)[1, 1], 0.05788217444, tolerance = 1e-7)
   expect_equal(dim(co[['pcaproject.l2']]), c(160, 30))
-  expect_equal(Embeddings(co[['pcaproject.l2']])[1, 1], 0.03807493471, tolerance = 1e-7)
+  expect_equal(Embeddings(co[['pcaproject.l2']])[1, 1], 0.04334884, tolerance = 1e-7)
   expect_equal(Loadings(co[['pcaproject.l2']], projected = T)[1, 1], 0.05788217444, tolerance = 1e-7)
   ref.cells <- paste0(Cells(ref), "_reference")
   query.cells <- paste0(Cells(query), "_query")
@@ -346,8 +352,8 @@ test_that("FindTransferAnchors with SCT and project.query work", {
   expect_equal(anchors@query.cells, query.cells)
   expect_equal(anchors@reference.objects, logical())
   anchor.mat <- anchors@anchors
-  expect_equal(dim(anchor.mat), c(288, 3))
-  expect_equal(as.vector(anchor.mat[1, ]), c(1, 1, 0.6138996139), tolerance = 1e-7)
+  expect_equal(dim(anchor.mat), c(290, 3))
+  expect_equal(as.vector(anchor.mat[1, ]), c(1, 1, 0.6315789), tolerance = 1e-7)
   expect_equal(max(anchor.mat[, 2]), 80)
   expect_null(anchors@offsets)
   expect_equal(length(anchors@anchor.features), 220)
@@ -363,7 +369,7 @@ test_that("FindTransferAnchors with SCT and l2.norm FALSE work", {
   expect_equal(Reductions(co), c("pcaproject"))
   expect_equal(DefaultAssay(co), "SCT")
   expect_equal(GetAssayData(co[["SCT"]])[1, 1], 0)
-  expect_equal(GetAssayData(co[["SCT"]], slot = "scale.data"), new("matrix"))
+  expect_equal(GetAssayData(co[["SCT"]], layer = "scale.data"), new("matrix"))
   expect_equal(dim(co[['pcaproject']]), c(160, 30))
   expect_equal(Embeddings(co[['pcaproject']])[1, 1], -1.852491719, tolerance = 1e-7)
   expect_equal(Loadings(co[['pcaproject']], projected = T)[1, 1], -0.1829401539, tolerance = 1e-7)
